#!/bin/bash
# This script upgrades antman

. /etc/init.d/functions.sh

ANTMAN_VERSION_TO_INSTALL=3.0.1a

NL=$'\n'
ARCH=`uname -m`
ANTSLE_WETTY_IMAGE="antsle/wetty:0.0.3"

if [ "$ARCH" == "aarch64" ]; then
  ANTSLE_WETTY_IMAGE="antsle/wetty-aarch64:1.0"
  sed -i 's+ExecStart=/usr/local/bin/antletsld+ExecStart=/usr/bin/python /usr/local/bin/antletsld+g' /etc/systemd/system/antletsl.service
  sed -i 's+OVSBOOTPROTO+BOOTPROTO+g' /etc/sysconfig/network-scripts/ifcfg-br0
  sed -e 's/dhclient br0//g' -i /etc/rc.d/rc.local
  systemctl daemon-reload
  systemctl start antletsl
  #restore log files
  mkdir -p /var/log/audit
  mkdir -p /var/log/chrony
  mkdir -p /var/log/libvirt
  mkdir -p /var/log/libvirt/qemu
  mkdir -p /var/log/libvirt/lxc
  mkdir -p /var/log/nginx
  mkdir -p /var/log/openvswitch
  mkdir -p /var/log/swtpm
  mkdir -p /var/log/tuned
fi

if grep -q "63ee60ad6d0b4373bea1c27485eb6673" /etc/machine-id
then
  rm -f /etc/machine-id /var/lib/dbus/machine-id
  dbus-uuidgen --ensure=/etc/machine-id
  dbus-uuidgen --ensure
fi

yum list installed lshw >/dev/null 2>&1
if [[ $? -ne 0 ]]; then
  yum install -y lshw
fi

autoaccept=false
while getopts ":y" opt; do
  case $opt in
    y)
      autoaccept=true
      ;;
    \?)
      echo "Invalid option: -$OPTARG" >&2
      exit 1
      ;;
    :)
      echo "Option -$OPTARG requires an argument." >&2
      exit 1
      ;;
  esac
done

vercomp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

get_json_variable() {
  variable=$1
  python -c "import sys, json; print json.load(sys.stdin)['$variable']"
}

aos=`antsleOS-version`
anthill_hostname="anthill.antsle.com"
anthill_protocol="https"
if [[ -f /etc/anthilld.conf ]]; then
  anthill_hostname=`cat /etc/anthilld.conf | get_json_variable 'hostname'`
  anthill_protocol=`cat /etc/anthilld.conf | get_json_variable 'protocol'`
fi
anthill_url="$anthill_protocol://$anthill_hostname"

ebegin "Checking if we're running as root"
if [ "${EUID}" -ne 0 ]
then
  eend 3 "Error: This script can only run as root. Aborting."
  exit 3
fi
eend 0

ebegin "Checking installed edgeLinux version"

if [[ "${aos}" != 2.*  ]]; then
  eend 2 "Error: This script can only run on edgeLinux 2.0.0. Installed edgeLinux version is ${aos}. Aborting."
  exit 2
fi
eend 0

eend 0 "Starting antman upgrade."

ebegin "About to install antman ${ANTMAN_VERSION_TO_INSTALL}"
if [ "$autoaccept" = false ]; then
  read -r -p "Continue? [y/N] " it
  it=${it,,}  # tolower ;-)
  if [[ ! $it =~ ^(yes|y)$ ]]
  then
    eend 1 "Aborting at user's request."
    exit 1
  fi
fi
eend 0 "Starting antman upgrade."

ebegin "Ensuring antman service is running"
service antman start || eend $? "service 'antman' failed to start, aborting." || exit $?
eend 0

timenow=`date +%Y-%m-%d-%H.%M.%S`

ebegin "Installing inlein"
wget https://github.com/hypirion/inlein/releases/download/0.2.0/inlein -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'inlein' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/inlein
eend 0

ebegin "Verifying ZFS filesystems"
zfs create antlets/_backups
eend 0

ebegin "Stopping antman service"
service antman stop || eend $? "service 'antman' failed to stop, aborting." || exit $?
eend 0

ebegin "Downloading antman.jar"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/antman.jar -N -P /usr/local/antsleOS/antman/target 2>/dev/null || eend $? "Download 'antman.jar' failed, aborting." || exit $?
eend 0

ebegin "Downloading antsle-vnc.tar.gz"
wget https://central.antsle.com/antsle-vnc-releases/0.1/antsle-vnc.tar.gz -N 2>/dev/null || eend $? "Download 'antsle-vnc.tar.gz' failed, aborting." || exit $?
rm -rf /usr/local/antsleOS/antsle-vnc
mkdir -p /usr/local/antsleOS/antsle-vnc
tar -C /usr/local/antsleOS/antsle-vnc -xvzf antsle-vnc.tar.gz >/dev/null || eend $? "Extracting 'antsle-vnc.tar.gz' failed, aborting." || exit $?
rm antsle-vnc.tar.gz
eend 0

if [ "$ARCH" == "x86_64" ]; then
  ebegin "Downloading +lxc.antlet.xml"
  wget https://static-files.antsle.com:8443/templates/+lxc.antlet.xml -N -P /usr/local/antsleOS/xml 2>/dev/null || eend $? "Download 'lxc.antlet.xml' failed, aborting." || exit $?
  eend 0
fi

ebegin "Installing antmand script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antmand -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antmand' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antmand
eend 0

ebegin "Installing create-antlet script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/create-antlet -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'create-antlet' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/create-antlet
eend 0

ebegin "Installing delete-antlet script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/delete-antlet -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'delete-antlet' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/delete-antlet
eend 0

ebegin "Installing antsle-auth script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsle-auth -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsle-auth' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsle-auth
eend 0

ebegin "Installing portmappings script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/portmappings -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'portmappings' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/portmappings
eend 0

ebegin "Installing get-private-ip script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/get-private-ip -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'get-private-ip' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/get-private-ip
eend 0

ebegin "Installing create-template script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/create-template -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'create-template' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/create-template
eend 0

ebegin "Installing run-antsle-checks script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/run-antsle-checks -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'run-antsle-checks' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/run-antsle-checks
eend 0

ebegin "Installing antsleOS-netconf script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsleOS-netconf -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsleOS-netconf' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsleOS-netconf
eend 0

ebegin "Installing antsle-utils script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsle-utils -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsle-utils' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsle-utils
eend 0

ebegin "Installing launch-novnc.sh script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/launch-novnc.sh -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'launch-novnc.sh' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/launch-novnc.sh
eend 0

ebegin "Installing antsleOS-logs script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsleOS-logs -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsleOS-logs' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsleOS-logs
eend 0

ebegin "Installing antsleOS-baseip script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsleOS-baseip -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsleOS-baseip' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsleOS-baseip
eend 0

ebegin "Installing antsleOS-backup script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsleOS-backup -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsleOS-backup' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsleOS-backup
eend 0

ebegin "Installing antsleOS-webconsole script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsleOS-webconsole -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsleOS-webconsole' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsleOS-webconsole
eend 0

ebegin "Installing name-antsle script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/name-antsle -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'name-antsle' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/name-antsle
eend 0

ebegin "Installing antsleOS-storage-info script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsleOS-storage-info -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsleOS-storage-info' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsleOS-storage-info
eend 0

ebegin "Installing zfsdeps script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/zfsdeps -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'zfsdeps' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/zfsdeps
eend 0

ebegin "Installing delete-antlet-force script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/delete-antlet-force -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'delete-antlet-force' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/delete-antlet-force
eend 0

# ebegin "Installing upgrade-anthilld script"
# wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/upgrade-anthilld -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'upgrade-anthilld' script failed, aborting." || exit $?
# chmod a+x /usr/local/bin/upgrade-anthilld
# eend 0

# ebegin "Installing upgrade-edgelinux script"
# wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/upgrade-edgelinux -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'upgrade-edgelinux' script failed, aborting." || exit $?
# chmod a+x /usr/local/bin/upgrade-edgelinux
# eend 0

ebegin "Installing import-antlet script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/import-antlet -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'import-antlet' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/import-antlet
eend 0

ebegin "Installing export-antlet script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/export-antlet -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'export-antlet' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/export-antlet
eend 0

ebegin "Installing anthilld"
rm -rf /var/log/edgeLinux/anthilld.*
upgrade-anthilld 1.3.0
eend 0

ebegin "Installing .m2"
wget https://static-files.antsle.com:8443/misc/m2.tar.gz 2>/dev/null || eend $? "Downloading m2.tar.gz failed, aborting." || exit $?
tar -zxf m2.tar.gz -C /root >/dev/null || eend $? "Extracting m2.tar.gz failed, aborting." || exit $?
rm m2.tar.gz
eend 0

ebegin "Installing antsleOS-antaid script"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/bin/antsleOS-antaid -N -P /usr/local/bin 2>/dev/null || eend $? "Installation of 'antsleOS-antaid' script failed, aborting." || exit $?
chmod a+x /usr/local/bin/antsleOS-antaid
eend 0

if [ -f /etc/nginx/virtualhosts/myantsle.local.conf ]; then
  ebegin "Backing up myantsle.local.conf"
  mkdir -p /etc/nginx/virtualhosts/.antsle-backups/
  cp /etc/nginx/virtualhosts/myantsle.local.conf /etc/nginx/virtualhosts/.antsle-backups/myantsle.local.conf.bk.${timenow} || eend $? "Backing up myantsle.local.conf failed, aborting." || exit $?
  eend 0
fi

if [ ! -f /etc/nginx/virtualhosts/antlet_name_to_ip.conf ]; then
  ebegin "Installing default antlet_name_to_ip.conf"
  echo 'map $host $host_to_antlet_ip {}' >/etc/nginx/virtualhosts/antlet_name_to_ip.conf
  eend 0
fi

ebegin "Downloading myantsle.local.conf nginx conf file"
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/os/etc/nginx/virtualhosts/myantsle.local.conf -N -P /tmp 2>/dev/null || eend $? "Downloading 'myantsle.local.conf' conf file failed, aborting." || exit $?
sed 's,{{antslename}},'`hostname`',g' /tmp/myantsle.local.conf >/etc/nginx/virtualhosts/myantsle.local.conf
rm /tmp/myantsle.local.conf
eend 0

ebegin "Installing console.myantsle.local.conf nginx conf file"
antsleOS-backup save /etc/nginx/virtualhosts/console.myantsle.local.conf
wget https://central.antsle.com/antman-releases/${ANTMAN_VERSION_TO_INSTALL}/os/etc/nginx/virtualhosts/console.myantsle.local.conf -N -P /tmp 2>/dev/null || eend $? "Downloading 'console.myantsle.local.conf' conf file failed, aborting." || exit $?
sed 's,{{antslename}},'`hostname`',g' /tmp/console.myantsle.local.conf >/etc/nginx/virtualhosts/console.myantsle.local.conf
rm /tmp/console.myantsle.local.conf
eend 0


# if crontab -l | grep run-antsle-checks >/dev/null; then
#   ebegin "Updating crontab"
#   crontab -l | perl -0pe "s|SHELL=/bin/bash\nPATH=/sbin:/bin:/usr/sbin:/usr/bin\n\* \* \* \* \* /usr/local/bin/run-antsle-checks|SHELL=/bin/bash${NL}PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/bin${NL}* * * * * /usr/local/bin/run-antsle-checks|g" | crontab - || eend $? "Could not update crontab, aborting." || exit $?
#   eend 0

#   ebegin "Installing run-antsle-checks script"
#   crontab -l | grep 'run-antsle-checks' || (echo "SHELL=/bin/bash${NL}PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/bin${NL}* * * * * /usr/local/bin/run-antsle-checks"; crontab -l) | crontab -
#   eend 0

#   ebegin "Installing run-rollback-if-due check"
#   crontab -l | grep 'antsleOS-netconf run-rollback-if-due' || (crontab -l 2>/dev/null; echo "* * * * * /usr/local/bin/antsleOS-netconf run-rollback-if-due";) | crontab -
#   eend 0
# else
#   crontab -l >/usr/local/antsleOS/backups/crontab/crontab.bk.${timenow} || eend $? "Could not backup crontab, aborting." || exit $?
#   echo "SHELL=/bin/bash${NL}PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/bin${NL}* * * * * /usr/local/bin/run-antsle-checks${NL}* * * * * /usr/local/bin/antsleOS-netconf run-rollback-if-due" | crontab -
#   einfo "Your crontab has been reset. A backup of the previous version can be found here: /usr/local/antsleOS/backups/crontab/crontab.bk.${timenow}"
# fi


if [[ "$ARCH" == "x86_64" ]]; then
  ebegin "Downloading libsigar-amd64-linux.so"
  wget https://central.antsle.com/sigar-bin/lib/libsigar-amd64-linux.so -N -P /usr/lib 2>/dev/null || eend $? "Download 'libsigar-amd64-linux.so' failed, aborting." || exit $?
  eend 0
  ebegin "Downloading log4j.jar"
  wget https://central.antsle.com/sigar-bin/lib/log4j.jar -N -P /usr/lib 2>/dev/null || eend $? "Download 'log4j.jar' failed, aborting." || exit $?
  eend 0
  ebegin "Downloading sigar.jar"
  wget https://central.antsle.com/sigar-bin/lib/sigar.jar -N -P /usr/lib 2>/dev/null || eend $? "Download 'sigar.jar' failed, aborting." || exit $?
  eend 0
fi


ebegin "Migrating openRC scripts..."
find /etc/init.d -type f -exec sed -i 's/runscript/openrc-run/' {} +
eend 0

if [[ `docker ps -a -f "name=antsle-wetty" --format '{{.Image}}'` != $ANTSLE_WETTY_IMAGE ]]; then
  ebegin "Installing wetty docker image"
  
  service docker start || eend $? "Could not start docker, aborting." || exit $?
  start_docker_tries=0
  start_docker_tries_max=10
  while [  $start_docker_tries -lt $start_docker_tries_max ]; do
    sleep 1
    docker ps >/dev/null 2>/dev/null
    if [[ $? -eq 0 ]]; then
      break
    fi
    echo "."
    let start_docker_tries=start_docker_tries+1 
  done

  docker ps >/dev/null 2>/dev/null
  if [[ $? -ne 0 ]]; then
    eend $? "Docker did not start after $start_docker_tries_max seconds. Please try again later."
    exit $?
  fi

  docker rm -f antsle-wetty >/dev/null 2>/dev/null
  docker create --name antsle-wetty -p 6822:3000 $ANTSLE_WETTY_IMAGE || eend $? "Could not download wetty docker image, aborting." || exit $?
  eend 0
fi

which docker-compose >/dev/null 2>/dev/null
if [[ $? -ne 0 ]]; then
  ebegin "Installing docker-compose..."
  curl -L https://github.com/docker/compose/releases/download/1.17.0/docker-compose-`uname -s`-`uname -m` -o /usr/local/bin/docker-compose || eend $? "Could not install docker-compose, aborting." || exit $?
  chmod +x /usr/local/bin/docker-compose || eend $? "Could not install docker-compose, aborting." || exit $?
  eend 0
fi

if cat /etc/ssh/sshd_config | grep "Match Group sftp" >/dev/null; then
  true
else
  ebegin "Updating sshd_config"
  mkdir -p /etc/ssh/.antsle-backups/
  cp /etc/ssh/sshd_config /etc/ssh/.antsle-backups/sshd_config.${timenow} >/dev/null
  (awk '/#GatewayPorts no/ {print "GatewayPorts yes";next;}; {print;}' /etc/ssh/.antsle-backups/sshd_config.${timenow} | \
  awk '/Subsystem\tsftp\t\/usr\/lib64\/misc\/sftp-server/ {print "Subsystem sftp internal-sftp";next;}; {print;}' >/etc/ssh/sshd_config) || eend $? "Could not write to /etc/ssh/sshd_config, aborting." || exit $?
  (echo "${NL}Match Group sftp$NL  ForceCommand internal-sftp$NL  AllowTcpForwarding no  $NL  PasswordAuthentication yes$NL" >>/etc/ssh/sshd_config) || eend $? "Could not write to /etc/ssh/sshd_config, aborting." || exit $?
  service sshd restart || eend $? "service 'sshd' failed to start, aborting." || exit $?
  eend 0
fi

ebegin "Starting antman service"
service antman start || eend $? "service 'antman' failed to start, aborting." || exit $?
eend 0

ebegin "Restarting nginx service"
service nginx restart || eend $? "service 'nginx' failed to restart, aborting." || exit $?
eend 0

einfo "Success, you are now on antman ${ANTMAN_VERSION_TO_INSTALL} :-)"

exit 0
